@@ -26,45 +26,38 @@
  ****************************************************************
  */
 
+#include "config.h"
+
+#include "fileio.h"
+
 #include <sys/types.h>
 #include <fcntl.h>
+#include <stdint.h>
+#include <stdbool.h>
 #include <sys/stat.h>
 #include <pwd.h>
 
-#ifndef SIGINT
-# include <signal.h>
-#endif
+#include <signal.h>
 
-#include "config.h" 
 #include "screen.h"
-#include "extern.h"
 
-extern struct display *display, *displays;
-extern struct win *fore;
-extern struct layer *flayer;
-extern int    ServerSocket;
-extern int    real_uid, eff_uid;
-extern int    real_gid, eff_gid;
-extern char  *extra_incap, *extra_outcap;
-extern char  *home, *RcFileName;
-extern char   SockPath[], *SockName;
-#ifdef COPY_PASTE
-extern char  *BufferFile;
-#endif
-extern int    hardcopy_append;
-extern char  *hardcopydir;
+#include "misc.h"
+#include "process.h"
+#include "termcap.h"
+#include "dumptermcap.h"
+#include "encoding.h"
 
-static char  *CatExtra __P((char *, char *));
-static char  *findrcfile __P((char *));
+static char *CatExtra(char *, char *);
+static char *findrcfile(char *);
 
 char *rc_name = "";
 int rc_recursion = 0;
 
-static char *
-CatExtra(register char *str1, register char *str2)
+static char *CatExtra(char *str1, char *str2)
 {
-	register char *cp;
-	register int len1, len2, add_colon;
+	char *cp;
+	size_t len1, len2;
+	bool add_colon;
 
 	len1 = strlen(str1);
 	if (len1 == 0)
@@ -72,22 +65,22 @@ CatExtra(register char *str1, register char *str2)
 	add_colon = (str1[len1 - 1] != ':');
 	if (str2) {
 		len2 = strlen(str2);
-		if ((cp = realloc(str2, (unsigned)len1 + len2 + add_colon + 1)) == NULL)
+		if ((cp = realloc(str2, len1 + len2 + add_colon + 1)) == NULL)
 			Panic(0, "%s", strnomem);
-		bcopy(cp, cp + len1 + add_colon, len2 + 1);
+		memmove(cp + len1 + add_colon, cp, len2 + 1);
 	} else {
-		if ((cp = malloc((unsigned)len1 + add_colon + 1)) == NULL)
+		if ((cp = malloc(len1 + add_colon + 1)) == NULL)
 			Panic(0, "%s", strnomem);
 		cp[len1 + add_colon] = '\0';
 	}
-	bcopy(str1, cp, len1);
+	memmove(cp, str1, len1);
 	if (add_colon)
 		cp[len1] = ':';
+
 	return cp;
 }
 
-static char *
-findrcfile(char *rcfile)
+static char *findrcfile(char *rcfile)
 {
 	char buf[256];
 	char *p;
@@ -97,9 +90,7 @@ findrcfile(char *rcfile)
 	if (rcfile && *rcfile == '~') {
 		static char rcfilename_tilde_exp[MAXPATHLEN + 1];
 		char *slash_position = strchr(rcfile, '/');
-
 		if (slash_position == rcfile + 1) {
-			char *home = getenv("HOME");
 			if (!home) {
 				Msg(0, "%s: source: tilde expansion failed", rc_name);
 				return NULL;
@@ -122,24 +113,19 @@ findrcfile(char *rcfile)
 	}
 
 	if (rcfile) {
-		char *rcend = rindex(rc_name, '/');
-		if (*rcfile != '/' && rcend && (rcend - rc_name) + strlen(rcfile) + 2 < sizeof(buf)) {
+		char *rcend = strrchr(rc_name, '/');
+		if (*rcfile != '/' && rcend && (rcend - rc_name) + strlen(rcfile) + 2 < ARRAY_SIZE(buf)) {
 			strncpy(buf, rc_name, rcend - rc_name + 1);
-			strcpy(buf + (rcend - rc_name) + 1, rcfile);
+			strncpy(buf + (rcend - rc_name) + 1, rcfile, 256 - (rcend - rc_name));
 			if (access(buf, R_OK) == 0)
 				return SaveStr(buf);
 		}
-		debug1("findrcfile: you specified '%s'\n", rcfile);
 		return SaveStr(rcfile);
 	}
-
-	debug("findrcfile: you specified nothing...\n");
 	if ((p = getenv("SCREENRC")) != NULL && *p != '\0') {
-		debug1("  $SCREENRC has: '%s'\n", p);
 		return SaveStr(p);
 	} else {
-		debug("  ...nothing in $SCREENRC, defaulting $HOME/.screenrc\n");
-		if (strlen(home) > sizeof(buf) - 12)
+		if (strlen(home) > ARRAY_SIZE(buf) - 12)
 			Panic(0, "Rc: home too large");
 		sprintf(buf, "%s/.screenrc", home);
 		return SaveStr(buf);
@@ -151,11 +137,10 @@ findrcfile(char *rcfile)
  * 1) rcfilename = "/etc/screenrc"
  * 2) rcfilename = RcFileName
  */
-int
-StartRc(char *rcfilename, int nopanic)
+int StartRc(char *rcfilename, int nopanic)
 {
-	register int argc, len;
-	register char *p, *cp;
+	int argc, len;
+	char *p, *cp;
 	char buf[2048];
 	char *args[MAXARGS];
 	int argl[MAXARGS];
@@ -167,36 +152,35 @@ StartRc(char *rcfilename, int nopanic)
 
 	/* Special settings for vt100 and others */
 	if (display && (!strncmp(D_termname, "vt", 2) || !strncmp(D_termname, "xterm", 5)))
-		extra_incap = CatExtra("xn:f0=\033Op:f1=\033Oq:f2=\033Or:f3=\033Os:f4=\033Ot:f5=\033Ou:f6=\033Ov:f7=\033Ow:f8=\033Ox:f9=\033Oy:f.=\033On:f,=\033Ol:fe=\033OM:f+=\033Ok:f-=\033Om:f*=\033Oj:f/=\033Oo:fq=\033OX", extra_incap);
+		extra_incap =
+		    CatExtra
+		    ("xn:f0=\033Op:f1=\033Oq:f2=\033Or:f3=\033Os:f4=\033Ot:f5=\033Ou:f6=\033Ov:f7=\033Ow:f8=\033Ox:f9=\033Oy:f.=\033On:f,=\033Ol:fe=\033OM:f+=\033Ok:f-=\033Om:f*=\033Oj:f/=\033Oo:fq=\033OX",
+		     extra_incap);
 
 	rc_name = findrcfile(rcfilename);
+
 	if (rc_name == NULL || (fp = secfopen(rc_name, "r")) == NULL) {
 		const char *rc_nonnull = rc_name ? rc_name : rcfilename;
 		if (!rc_recursion && RcFileName && !strcmp(RcFileName, rc_nonnull)) {
-		/*
-		 * User explicitly gave us that name,
-		 * this is the only case, where we get angry, if we can't read
-		 * the file.
-		 */
-			debug3("StartRc: '%s','%s', '%s'\n", RcFileName, rc_name ? rc_name : "(null)", rcfilename);
+			/*
+			 * User explicitly gave us that name,
+			 * this is the only case, where we get angry, if we can't read
+			 * the file.
+			 */
 			if (!nopanic)
 				Panic(0, "Unable to open \"%s\".", rc_nonnull);
-				/* possibly NOTREACHED */
+			/* possibly NOTREACHED */
 		}
-
-		debug1("StartRc: '%s' no good. ignored\n", rc_nonnull);
 		if (rc_name)
 			Free(rc_name);
 		rc_name = oldrc_name;
 		return 1;
 	}
-	while (fgets(buf, sizeof buf, fp) != NULL) {
-		if ((p = rindex(buf, '\n')) != NULL)
+	while (fgets(buf, ARRAY_SIZE(buf), fp) != NULL) {
+		if ((p = strrchr(buf, '\n')) != NULL)
 			*p = '\0';
-
-		if ((argc = Parse(buf, sizeof buf, args, argl)) == 0)
+		if ((argc = Parse(buf, ARRAY_SIZE(buf), args, argl)) == 0)
 			continue;
-
 		if (strcmp(args[0], "echo") == 0) {
 			if (!display)
 				continue;
@@ -212,27 +196,22 @@ StartRc(char *rcfilename, int nopanic)
 		} else if (strcmp(args[0], "sleep") == 0) {
 			if (!display)
 				continue;
-			debug("sleeeeeeep\n");
 			if (argc != 2) {
 				Msg(0, "%s: sleep: one numeric argument expected.", rc_name);
 				continue;
 			}
 			DisplaySleep1000(1000 * atoi(args[1]), 1);
 		}
-#ifdef TERMINFO
-    else if (!strcmp(args[0], "termcapinfo") || !strcmp(args[0], "terminfo")) {
-#else
-		else if (!strcmp(args[0], "termcapinfo") || !strcmp(args[0], "termcap")) {
-#endif
+		else if (!strcmp(args[0], "termcapinfo") || !strcmp(args[0], "terminfo"))
+		{
 			if (!display)
 				continue;
 			if (argc < 3 || argc > 4) {
 				Msg(0, "%s: %s: incorrect number of arguments.", rc_name, args[0]);
 				continue;
 			}
-
 			for (p = args[1]; p && *p; p = cp) {
-				if ((cp = index(p, '|')) != 0)
+				if ((cp = strchr(p, '|')) != NULL)
 					*cp++ = '\0';
 				len = strlen(p);
 				if (p[len - 1] == '*') {
@@ -260,8 +239,7 @@ StartRc(char *rcfilename, int nopanic)
 	return 0;
 }
 
-void
-FinishRc(char *rcfilename)
+void FinishRc(char *rcfilename)
 {
 	char buf[2048];
 	FILE *fp;
@@ -274,32 +252,28 @@ FinishRc(char *rcfilename)
 		if (rc_recursion)
 			Msg(errno, "%s: source %s", oldrc_name, rc_nonnull);
 		else if (RcFileName && !strcmp(RcFileName, rc_nonnull)) {
-      /*
-       * User explicitly gave us that name,
-       * this is the only case, where we get angry, if we can't read
-       * the file.
-       */
-			debug3("FinishRc:'%s','%s','%s'\n", RcFileName, rc_name ? rc_name : "(null)", rcfilename);
+			/*
+			 * User explicitly gave us that name,
+			 * this is the only case, where we get angry, if we can't read
+			 * the file.
+			 */
 			Panic(0, "Unable to open \"%s\".", rc_nonnull);
 			/* NOTREACHED */
 		}
-		debug1("FinishRc: '%s' no good. ignored\n", rc_nonnull);
 		if (rc_name)
 			Free(rc_name);
 		rc_name = oldrc_name;
 		return;
 	}
 
-	debug("finishrc is going...\n");
-	while (fgets(buf, sizeof buf, fp) != NULL)
-		RcLine(buf, sizeof buf);
+	while (fgets(buf, ARRAY_SIZE(buf), fp) != NULL)
+		RcLine(buf, ARRAY_SIZE(buf));
 	(void)fclose(fp);
 	Free(rc_name);
 	rc_name = oldrc_name;
 }
 
-void
-do_source(char *rcfilename)
+void do_source(char *rcfilename)
 {
 	if (rc_recursion > 10) {
 		Msg(0, "%s: source: recursion limit reached", rc_name);
@@ -315,82 +289,77 @@ do_source(char *rcfilename)
  * The fore window is taken from the display as well as the user.
  * This is bad when we run detached.
  */
-void
-RcLine(char *ubuf, int ubufl)
+void RcLine(char *ubuf, int ubufl)
 {
 	char *args[MAXARGS];
 	int argl[MAXARGS];
-#ifdef MULTIUSER
-	extern struct acluser *EffectiveAclUser;      /* acl.c */
-	extern struct acluser *users;                 /* acl.c */
-#endif
 
 	if (display) {
 		fore = D_fore;
 		flayer = D_forecv->c_layer;
 	} else
-		flayer = fore ? fore->w_savelayer : 0;
+		flayer = fore ? fore->w_savelayer : NULL;
 	if (Parse(ubuf, ubufl, args, argl) <= 0)
 		return;
-
-#ifdef MULTIUSER
 	if (!display) {
-      /* the session owner does it, when there is no display here */
+		/* the session owner does it, when there is no display here */
 		EffectiveAclUser = users;
-		debug("RcLine: WARNING, no display no user! Session owner executes command\n");
 	}
-#endif
-
 	DoCommand(args, argl);
-
-#ifdef MULTIUSER
-	EffectiveAclUser = 0;
-#endif
+	EffectiveAclUser = NULL;
 }
 
-/*  needs display for copybuffer access and termcap dumping  */
-void
-WriteFile(struct acluser *user, char *fn, int dump)
-{
-  /* dump==0:	create .termcap,
-   * dump==1:	hardcopy,
-   * #ifdef COPY_PASTE
-   * dump==2:	BUFFERFILE
-   * #endif COPY_PASTE
-   * dump==1:	scrollback,
-   */
-	register int i, j, k;
-	register char *p;
-	register FILE *f;
-	char fnbuf[1024];
-	char *mode = "w";
 
-#ifdef COPY_PASTE
+static void putc_encoded(FILE *f, uint32_t c, uint32_t font, int encoding) {
+	/* Filler character which is used for double-width characters. */
+	if (c == 0xff && font == 0xff)
+		return;
+
+	char buf[10];
+	int length = EncodeChar(buf, c, encoding, NULL);
+	if (length < 0) {
+		return;
+	}
+	buf[length] = 0;
+	fputs(buf, f);
+}
+
+/*
+ * needs display for copybuffer access and termcap dumping
+ */
+void WriteFile(struct acluser *user, char *fn, int dump)
+{
+	/* dump==0:   create .termcap,
+	 * dump==1:   hardcopy,
+	 * dump==2:   BUFFERFILE
+	 * dump==1:   scrollback,
+	 */
+	int i, j, k;
+	char *c;
+	FILE *f;
+	char fnbuf[FILENAME_MAX];
+	char *mode = "w";
 	int public = 0;
-# ifdef HAVE_LSTAT
 	struct stat stb, stb2;
 	int fd, exists = 0;
-# endif
-#endif
 
 	switch (dump) {
 	case DUMP_TERMCAP:
-		if (fn == 0) {
-			i = SockName - SockPath;
-			if (i > (int)sizeof(fnbuf) - 9)
+		if (fn == NULL) {
+			i = SocketName - SocketPath;
+			if (i > (int)ARRAY_SIZE(fnbuf) - 9)
 				i = 0;
-			strncpy(fnbuf, SockPath, i);
-			strcpy(fnbuf + i, ".termcap");
+			strncpy(fnbuf, SocketPath, i);
+			strncpy(fnbuf + i, ".termcap", 9);
 			fn = fnbuf;
 		}
 		break;
-
 	case DUMP_HARDCOPY:
 	case DUMP_SCROLLBACK:
-		if (fn == 0) {
-			if (fore == 0)
+		if (fn == NULL) {
+			if (fore == NULL)
 				return;
-			if (hardcopydir && *hardcopydir && strlen(hardcopydir) < sizeof(fnbuf) - 21)
+			if (hardcopydir && *hardcopydir && strlen(hardcopydir) < ARRAY_SIZE(fnbuf) - 21)
 				sprintf(fnbuf, "%s/hardcopy.%d", hardcopydir, fore->w_number);
 			else
 				sprintf(fnbuf, "hardcopy.%d", fore->w_number);
@@ -399,54 +368,46 @@ WriteFile(struct acluser *user, char *fn, int dump)
 		if (hardcopy_append && !access(fn, W_OK))
 			mode = "a";
 		break;
-
-#ifdef COPY_PASTE
 	case DUMP_EXCHANGE:
-		if (fn == 0) {
-			strncpy(fnbuf, BufferFile, sizeof(fnbuf) - 1);
-			fnbuf[sizeof(fnbuf) - 1] = 0;
+		if (fn == NULL) {
+			strncpy(fnbuf, BufferFile, ARRAY_SIZE(fnbuf) - 1);
+			fnbuf[ARRAY_SIZE(fnbuf) - 1] = 0;
 			fn = fnbuf;
 		}
 		public = !strcmp(fn, DEFAULT_BUFFERFILE);
-# ifdef HAVE_LSTAT
 		exists = !lstat(fn, &stb);
 		if (public && exists && (S_ISLNK(stb.st_mode) || stb.st_nlink > 1)) {
 			Msg(0, "No write to links, please.");
 			return;
 		}
-# endif
 		break;
-#endif
 	}
 
-	debug2("WriteFile(%d) %s\n", dump, fn);
 	if (UserContext() > 0) {
-		debug("Writefile: usercontext\n");
-#ifdef COPY_PASTE
 		if (dump == DUMP_EXCHANGE && public) {
-# ifdef HAVE_LSTAT
 			if (exists) {
 				if ((fd = open(fn, O_WRONLY, 0666)) >= 0) {
-					if (fstat(fd, &stb2) == 0 && stb.st_dev == stb2.st_dev && stb.st_ino == stb2.st_ino)
-						ftruncate(fd, 0);
-					else {
+					if (fstat(fd, &stb2) == 0
+					    && stb.st_dev == stb2.st_dev
+					    && stb.st_ino == stb2.st_ino) {
+						if (ftruncate(fd, 0) != 0) {
+							close(fd);
+							fd = -1;
+						}
+					} else {
 						close(fd);
 						fd = -1;
 					}
 				}
 			} else
-				fd = open(fn, O_WRONLY|O_CREAT|O_EXCL, 0666);
-			f = fd >= 0 ? fdopen(fd, mode) : 0;
-# else
-			f = fopen(fn, mode);
-# endif
+				fd = open(fn, O_WRONLY | O_CREAT | O_EXCL, 0666);
+			f = fd >= 0 ? fdopen(fd, mode) : NULL;
 		} else
-#endif /* COPY_PASTE */
 			f = fopen(fn, mode);
 		if (f == NULL) {
-			debug2("WriteFile: fopen(%s,\"%s\") failed\n", fn, mode);
 			UserReturn(0);
 		} else {
+			uint32_t *p, *pf;
 			switch (dump) {
 			case DUMP_HARDCOPY:
 			case DUMP_SCROLLBACK:
@@ -459,43 +420,35 @@ WriteFile(struct acluser *user, char *fn, int dump)
 					fputs("<\n", f);
 				}
 				if (dump == DUMP_SCROLLBACK) {
-#ifdef COPY_PASTE
 					for (i = fore->w_histheight - fore->w_scrollback_height; i < fore->w_histheight; i++) {
-						p = (char *)(WIN(i)->image);
-						for (k = fore->w_width - 1;
-						    k >= 0 && p[k] == ' '; k--)
-							;
+						p = (WIN(i)->image);
+						pf = WIN(i)->font;
+						for (k = fore->w_width - 1; k >= 0 && p[k] == ' '; k--) ;
 						for (j = 0; j <= k; j++)
-							putc(p[j], f);
+							putc_encoded(f, p[j], pf[j], fore->w_encoding);
 						putc('\n', f);
 					}
-#endif
 				}
 				for (i = 0; i < fore->w_height; i++) {
-					p = (char *)fore->w_mlines[i].image;
-					for (k = fore->w_width - 1;
-					    k >= 0 && p[k] == ' '; k--)
-						;
+					p = fore->w_mlines[i].image;
+					pf = fore->w_mlines[i].font;
+					for (k = fore->w_width - 1; k >= 0 && p[k] == ' '; k--) ;
 					for (j = 0; j <= k; j++)
-						putc(p[j], f);
+						putc_encoded(f, p[j], pf[j], fore->w_encoding);
 					putc('\n', f);
 				}
 				break;
-
 			case DUMP_TERMCAP:
 				DumpTermcap(fore->w_aflag, f);
 				break;
-
-#ifdef COPY_PASTE
 			case DUMP_EXCHANGE:
-				p = user->u_plop.buf;
-				for (i = user->u_plop.len; i-- > 0; p++)
-					if (*p == '\r' && (i == 0 || p[1] != '\n'))
+				c = user->u_plop.buf;
+				for (i = user->u_plop.len; i-- > 0; c++)
+					if (*c == '\r' && (i == 0 || c[1] != '\n'))
 						putc('\n', f);
 					else
-						putc(*p, f);
+						putc(*c, f);
 				break;
-#endif
 			}
 			(void)fclose(f);
 			UserReturn(1);
@@ -512,196 +465,98 @@ WriteFile(struct acluser *user, char *fn, int dump)
 		case DUMP_SCROLLBACK:
 			Msg(0, "Screen image %s to \"%s\".", (*mode == 'a') ? "appended" : "written", fn);
 			break;
-#ifdef COPY_PASTE
 		case DUMP_EXCHANGE:
 			Msg(0, "Copybuffer written to \"%s\".", fn);
-#endif
 		}
 	}
 }
 
-#ifdef COPY_PASTE
-
 /*
- * returns an allocated buffer which holds a copy of the file named fn.
+ * returns an allocated buffer which holds a copy of the file named filename.
  * lenp (if nonzero) points to a location, where the buffer size should be
  * stored.
  */
-char *
-ReadFile(char *fn, int *lenp)
+char *ReadFile(char *filename, int *lenp)
 {
-	int i, l, size;
-	char c, *bp, *buf;
-	struct stat stb;
+	FILE *file;
+	int l, size;
+	char *buf = NULL;
 
-	ASSERT(lenp);
-	debug1("ReadFile(%s)\n", fn);
-
-	if ((i = secopen(fn, O_RDONLY, 0)) < 0) {
-		Msg(errno, "no %s -- no slurp", fn);
+	if ((file = secfopen(filename, "r")) == NULL) {
+		Msg(errno, "no %s -- no slurp", filename);
 		return NULL;
 	}
 
-	if (fstat(i, &stb)) {
-		Msg(errno, "no good %s -- no slurp", fn);
-		close(i);
+	if (fseek(file, 0L, SEEK_END) < 0) {
+		fclose(file);
+		Msg(errno, "fseek %s", filename);
 		return NULL;
 	}
-	size = stb.st_size;
 
+	size = ftell(file);
+	if (size < 0) {
+		fclose(file);
+		Msg(errno, "ftell %s", filename);
+		return NULL;
+	}
 	if ((buf = malloc(size)) == NULL) {
-		close(i);
+		fclose(file);
 		Msg(0, "%s", strnomem);
 		return NULL;
 	}
+	if (fseek(file, 0L, SEEK_SET) < 0) {
+		free(buf);
+		fclose(file);
+		Msg(errno, "fseek %s", filename);
+		return NULL;
+	}
 	errno = 0;
 
-	if ((l = read(i, buf, size)) != size) {
-		if (l < 0)
-			l = 0;
-		Msg(errno, "Got only %d bytes from %s", l, fn);
-	} else {
-		if (read(i, &c, 1) > 0)
-			Msg(0, "Slurped only %d characters (of %d) into buffer - try again", l, size);
-		else
-			Msg(0, "Slurped %d characters into buffer", l);
+	if ((l = fread(buf, sizeof(char), size, file)) != size) {
+		Msg(errno, "Got only %d bytes from %s", l, filename);
 	}
-	close(i);
+	fclose(file);
 	*lenp = l;
-	for (bp = buf; l-- > 0; bp++)
-		if (*bp == '\n' && (bp == buf || bp[-1] != '\r'))
-			*bp = '\r';
 	return buf;
 }
 
-void
-KillBuffers()
+void KillBuffers(void)
 {
 	if (UserContext() > 0)
 		UserReturn(unlink(BufferFile) ? errno : 0);
 	errno = UserStatus();
 	Msg(errno, "%s %sremoved", BufferFile, errno ? "not " : "");
 }
-#endif	/* COPY_PASTE */
 
+/*
+ * (Almost) secure open and fopen...
+ */
 
-/* (Almost) secure open and fopen...  */
-
-FILE *
-secfopen(char *name, char *mode)
+FILE *secfopen(char *name, char *mode)
 {
 	FILE *fi;
-#ifndef USE_SETEUID
-	int flags, fd;
-#endif
 
-	debug2("secfopen(%s, %s)\n", name, mode);
-#ifdef USE_SETEUID
 	xseteuid(real_uid);
 	xsetegid(real_gid);
 	fi = fopen(name, mode);
 	xseteuid(eff_uid);
 	xsetegid(eff_gid);
 	return fi;
-#else
-	if (eff_uid == real_uid)
-		return fopen(name, mode);
-	if (mode[0] && mode[1] == '+')
-		flags = O_RDWR;
-	else
-		flags = (mode[0] == 'r') ? O_RDONLY : O_WRONLY;
-	if (mode[0] == 'w')
-		flags |= O_CREAT | O_TRUNC;
-	else if (mode[0] == 'a')
-		flags |= O_CREAT | O_APPEND;
-	else if (mode[0] != 'r') {
-		errno = EINVAL;
-		return 0;
-	}
-	if ((fd = secopen(name, flags, 0666)) < 0)
-		return 0;
-	if ((fi = fdopen(fd, mode)) == 0) {
-		close(fd);
-		return 0;
-	}
-	return fi;
-#endif
 }
 
-int
-secopen(char *name, int flags, int mode)
+int secopen(char *name, int flags, int mode)
 {
 	int fd;
-#ifndef USE_SETEUID
-	int q;
-	struct stat stb;
-#endif
 
-	debug3("secopen(%s, 0x%x, 0%03o)\n", name, flags, mode);
-#ifdef USE_SETEUID
 	xseteuid(real_uid);
 	xsetegid(real_gid);
 	fd = open(name, flags, mode);
 	xseteuid(eff_uid);
 	xsetegid(eff_gid);
 	return fd;
-#else
-	if (eff_uid == real_uid)
-		return open(name, flags, mode);
-
-	/* Truncation/creation is done in UserContext */
-	if ((flags & O_TRUNC) || ((flags & O_CREAT) && access(name, F_OK))) {
-		if (UserContext() > 0) {
-			if ((fd = open(name, flags, mode)) >= 0) {
-				close(fd);
-				UserReturn(0);
-			}
-			if (errno == 0)
-				errno = EACCES;
-			UserReturn(errno);
-		}
-		if ((q = UserStatus())) {
-			if (q > 0)
-				errno = q;
-			return -1;
-		}
-	}
-	if (access(name, F_OK))
-		return -1;
-	if ((fd = open(name, flags & ~(O_TRUNC | O_CREAT), 0)) < 0)
-		return -1;
-	debug("open successful\n");
-	if (fstat(fd, &stb)) {
-		close(fd);
-		return -1;
-	}
-	debug("fstat successful\n");
-	if (stb.st_uid != real_uid) {
-		switch (flags & (O_RDONLY | O_WRONLY | O_RDWR)) {
-		case O_RDONLY:
-			q = 0004;
-			break;
-		case O_WRONLY:
-			q = 0002;
-			break;
-		default:
-			q = 0006;
-			break;
-		}
-		if ((stb.st_mode & q) != q) {
-			debug1("secopen: permission denied (%03o)\n", stb.st_mode & 07777);
-			close(fd);
-			errno = EACCES;
-			return -1;
-		}
-	}
-	debug1("secopen ok - returning %d\n", fd);
-	return fd;
-#endif
 }
 
-int
-printpipe(struct win *p, char *cmd)
+int printpipe(Window *p, char *cmd)
 {
 	int pi[2];
 	if (pipe(pi)) {
@@ -714,24 +569,20 @@ printpipe(struct win *p, char *cmd)
 		return -1;
 	case 0:
 		display = p->w_pdisplay;
-		displays = 0;
+		displays = NULL;
 		ServerSocket = -1;
-#ifdef DEBUG
-		if (dfp && dfp != stderr)
-			fclose(dfp);
-#endif
 		close(0);
-		dup(pi[0]);
+		if (dup(pi[0]) < 0)
+			Panic(errno, "printpipe dup");
 		closeallfiles(0);
 		if (setgid(real_gid) || setuid(real_uid))
 			Panic(errno, "printpipe setuid");
 		eff_uid = real_uid;
 		eff_gid = real_gid;
-
 #ifdef SIGPIPE
-		signal(SIGPIPE, SIG_DFL);
+		xsignal(SIGPIPE, SIG_DFL);
 #endif
-		execl("/bin/sh", "sh", "-c", cmd, (char *)0);
+		execl("/bin/sh", "sh", "-c", cmd, NULL);
 		Panic(errno, "/bin/sh");
 	default:
 		break;
@@ -740,8 +591,7 @@ printpipe(struct win *p, char *cmd)
 	return pi[1];
 }
 
-int
-readpipe(char **cmdv)
+int readpipe(char **cmdv)
 {
 	int pi[2];
 
@@ -749,34 +599,25 @@ readpipe(char **cmdv)
 		Msg(errno, "pipe");
 		return -1;
 	}
-
 	switch (fork()) {
 	case -1:
 		Msg(errno, "fork");
 		return -1;
 	case 0:
-		displays = 0;
+		displays = NULL;
 		ServerSocket = -1;
-#ifdef DEBUG
-		if (dfp && dfp != stderr)
-			fclose(dfp);
-#endif
 		close(1);
 		if (dup(pi[1]) != 1) {
 			close(pi[1]);
 			Panic(0, "dup");
 		}
 		closeallfiles(1);
-
 		if (setgid(real_gid) || setuid(real_uid)) {
 			close(1);
 			Panic(errno, "setuid/setgid");
 		}
 		eff_uid = real_uid;
 		eff_gid = real_gid;
-#ifdef SIGPIPE
-		signal(SIGPIPE, SIG_DFL);
-#endif
 		execvp(*cmdv, cmdv);
 		close(1);
 		Panic(errno, "%s", *cmdv);
